package com.chelvc.cloud.admin.controller;

import javax.validation.Valid;
import javax.validation.constraints.Min;

import com.chelvc.cloud.uc.api.dto.ResourceDTO;
import com.chelvc.cloud.uc.api.param.ResourceModifyParam;
import com.chelvc.cloud.uc.api.param.ResourcePagingParam;
import com.chelvc.cloud.uc.api.service.ResourceService;
import com.chelvc.framework.base.annotation.UnifiedResponseBody;
import com.chelvc.framework.base.model.Pagination;
import com.chelvc.framework.base.util.ErrorUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 资源接口
 *
 * @author Woody
 * @date 2021/10/20
 */
@Validated
@RestController
@UnifiedResponseBody
@PreAuthorize("isType('ADMIN')")
public class ResourceController {
    @DubboReference
    private ResourceService resourceService;

    /**
     * 新增资源
     *
     * @param param 新增参数
     * @return 资源主键
     */
    @PostMapping("/resource")
    public Long addResource(@RequestBody @Valid ResourceModifyParam param) {
        return this.resourceService.addResource(param);
    }

    /**
     * 修改资源
     *
     * @param id    资源主键
     * @param param 修改参数
     */
    @PutMapping("/resource/{id}")
    public void updateResource(@PathVariable("id") @Min(value = 1, message = "资源主键不能小于1") Long id,
                               @RequestBody @Valid ResourceModifyParam param) {
        this.resourceService.updateResource(id, param);
    }

    /**
     * 获取资源信息
     *
     * @param id 资源主键
     * @return 资源信息
     */
    @GetMapping("/resource/{id}")
    public ResourceDTO getResource(@PathVariable("id") @Min(value = 1, message = "资源主键不能小于1") Long id) {
        ResourceDTO resource = this.resourceService.getResource(id);
        ErrorUtils.requireResource(resource, "资源不存在");
        return resource;
    }

    /**
     * 查询资源分页
     *
     * @param param 查询参数
     * @return 资源分页信息
     */
    @GetMapping("/resource/paging")
    public Pagination<ResourceDTO> getResourcePaging(@Valid ResourcePagingParam param) {
        return this.resourceService.getResourcePaging(param);
    }
}
